"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNodesGraph = exports.isNumber = exports.getNodeTypeForName = void 0;
const LoggerProxy_1 = require("./LoggerProxy");
const STICKY_NODE_TYPE = 'n8n-nodes-base.stickyNote';
function getNodeTypeForName(workflow, nodeName) {
    return workflow.nodes.find((node) => node.name === nodeName);
}
exports.getNodeTypeForName = getNodeTypeForName;
function isNumber(value) {
    return typeof value === 'number';
}
exports.isNumber = isNumber;
function getStickyDimensions(note, stickyType) {
    const heightProperty = stickyType === null || stickyType === void 0 ? void 0 : stickyType.description.properties.find((property) => property.name === 'height');
    const widthProperty = stickyType === null || stickyType === void 0 ? void 0 : stickyType.description.properties.find((property) => property.name === 'width');
    const defaultHeight = heightProperty && isNumber(heightProperty === null || heightProperty === void 0 ? void 0 : heightProperty.default) ? heightProperty.default : 0;
    const defaultWidth = widthProperty && isNumber(widthProperty === null || widthProperty === void 0 ? void 0 : widthProperty.default) ? widthProperty.default : 0;
    const height = isNumber(note.parameters.height) ? note.parameters.height : defaultHeight;
    const width = isNumber(note.parameters.width) ? note.parameters.width : defaultWidth;
    return {
        height,
        width,
    };
}
function areOverlapping(topLeft, bottomRight, targetPos) {
    return (targetPos[0] > topLeft[0] &&
        targetPos[1] > topLeft[1] &&
        targetPos[0] < bottomRight[0] &&
        targetPos[1] < bottomRight[1]);
}
function generateNodesGraph(workflow, nodeTypes) {
    var _a;
    const nodesGraph = {
        node_types: [],
        node_connections: [],
        nodes: {},
        notes: {},
    };
    const nodeNameAndIndex = {};
    try {
        const notes = workflow.nodes.filter((node) => node.type === STICKY_NODE_TYPE);
        const otherNodes = workflow.nodes.filter((node) => node.type !== STICKY_NODE_TYPE);
        notes.forEach((stickyNote, index) => {
            const stickyType = nodeTypes.getByNameAndVersion(STICKY_NODE_TYPE, stickyNote.typeVersion);
            const { height, width } = getStickyDimensions(stickyNote, stickyType);
            const topLeft = stickyNote.position;
            const bottomRight = [topLeft[0] + width, topLeft[1] + height];
            const overlapping = Boolean(otherNodes.find((node) => areOverlapping(topLeft, bottomRight, node.position)));
            nodesGraph.notes[index] = {
                overlapping,
                position: topLeft,
                height,
                width,
            };
        });
        otherNodes.forEach((node, index) => {
            var _a, _b, _c, _d, _e, _f;
            nodesGraph.node_types.push(node.type);
            const nodeItem = {
                type: node.type,
                position: node.position,
            };
            if (node.type === 'n8n-nodes-base.httpRequest') {
                try {
                    nodeItem.domain = new URL(node.parameters.url).hostname;
                }
                catch (e) {
                    nodeItem.domain = node.parameters.url;
                }
            }
            else {
                const nodeType = nodeTypes.getByNameAndVersion(node.type);
                nodeType === null || nodeType === void 0 ? void 0 : nodeType.description.properties.forEach((property) => {
                    if (property.name === 'operation' ||
                        property.name === 'resource' ||
                        property.name === 'mode') {
                        nodeItem[property.name] = property.default ? property.default.toString() : undefined;
                    }
                });
                nodeItem.operation = (_b = (_a = node.parameters.operation) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : nodeItem.operation;
                nodeItem.resource = (_d = (_c = node.parameters.resource) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : nodeItem.resource;
                nodeItem.mode = (_f = (_e = node.parameters.mode) === null || _e === void 0 ? void 0 : _e.toString()) !== null && _f !== void 0 ? _f : nodeItem.mode;
            }
            nodesGraph.nodes[`${index}`] = nodeItem;
            nodeNameAndIndex[node.name] = index.toString();
        });
        const getGraphConnectionItem = (startNode, connectionItem) => {
            return { start: nodeNameAndIndex[startNode], end: nodeNameAndIndex[connectionItem.node] };
        };
        Object.keys(workflow.connections).forEach((nodeName) => {
            const connections = workflow.connections[nodeName];
            connections.main.forEach((element) => {
                element.forEach((element2) => {
                    nodesGraph.node_connections.push(getGraphConnectionItem(nodeName, element2));
                });
            });
        });
    }
    catch (e) {
        const logger = (0, LoggerProxy_1.getInstance)();
        logger.warn(`Failed to generate nodes graph for workflowId: ${workflow.id}`);
        logger.warn(e.message);
        logger.warn((_a = e.stack) !== null && _a !== void 0 ? _a : '');
    }
    return { nodeGraph: nodesGraph, nameIndices: nodeNameAndIndex };
}
exports.generateNodesGraph = generateNodesGraph;
//# sourceMappingURL=TelemetryHelpers.js.map